@@ -1,4 +1,4 @@
-/* $Id: actions.cc 733 2007-05-02 20:19:32Z helly $ */
+/* $Id: actions.cc 750 2007-05-06 22:55:26Z helly $ */
 #include <time.h>
 #include <string.h>
 #include <iostream>
@@ -900,6 +900,11 @@ RuleOp::RuleOp(RegExp *e, RegExp *c, Token *t, uint a)
 	;
 }
 
+RuleOp* RuleOp::copy(uint a) const
+{
+	return new RuleOp(exp, ctx, new Token(code->text, code->line), a);
+}
+
 void RuleOp::calcSize(Char *rep)
 {
 	exp->calcSize(rep);
@@ -973,11 +978,6 @@ void optimize(Ins *i)
 	}
 }
 
-void genCode(std::ostream& o, RegExp *re)
-{
-	genCode(o, 0, re);
-}
-
 CharSet::CharSet()
 	: fix(0)
 	, freeHead(0)
@@ -1004,7 +1004,7 @@ CharSet::~CharSet()
 	delete[] ptn;
 }
 
-void genCode(std::ostream& o, uint ind, RegExp *re)
+void genCode(std::ostream& o, uint& ind, RegExp *re, const RegExpMap* specMap, const std::string& condName, bool isLastCond, bool &bPrologBrace)
 {
 	CharSet cs;
 	uint j;
@@ -1052,7 +1052,7 @@ void genCode(std::ostream& o, uint ind, RegExp *re)
 	}
 
 	DFA *dfa = new DFA(ins, re->size, 0, nRealChars, rep);
-	dfa->emit(o, ind);
+	dfa->emit(o, ind, specMap, condName, isLastCond, bPrologBrace);
 	delete dfa;
 	delete [] ins;
 	delete [] rep;
