@@ -1,9 +1,11 @@
-/* $Id: code.cc 733 2007-05-02 20:19:32Z helly $ */
+/* $Id: code.cc 750 2007-05-06 22:55:26Z helly $ */
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
 #include <iomanip>
 #include <iostream>
+#include <sstream>
+#include <time.h>
 #include "substr.h"
 #include "globals.h"
 #include "dfa.h"
@@ -16,7 +18,7 @@ namespace re2c
 // there must be at least one span in list;  all spans must cover
 // same range
 
-std::string indent(uint ind)
+static std::string indent(uint ind)
 {
 	std::string str;
 
@@ -27,6 +29,81 @@ std::string indent(uint ind)
 	return str;
 }
 
+template<typename _Ty>
+std::string replaceParam(std::string str, const std::string& param, const _Ty& value)
+{
+	std::ostringstream strValue;
+
+	strValue << value;
+
+	std::string::size_type pos;
+
+	while((pos = str.find(param)) != std::string::npos)
+	{
+		str.replace(pos, param.length(), strValue.str());
+	}
+
+	return str;
+}
+
+static void genYYFill(std::ostream &o, uint ind, uint need)
+{
+	if (bUseYYFillParam)
+	{
+		o << mapCodeName["YYFILL"];
+		if (!bUseYYFillNaked)
+		{
+			o << "(" << need << ");";
+		}
+		o << "\n";
+	}
+	else
+	{
+		o << replaceParam(mapCodeName["YYFILL"], yyFillLength, need);
+		if (!bUseYYFillNaked)
+		{
+			o << ";";
+		}
+		o << "\n";
+	}
+}
+
+static std::string genGetState()
+{
+	if (bUseYYGetStateNaked)
+	{
+		return mapCodeName["YYGETSTATE"];
+	}
+	else
+	{
+		return mapCodeName["YYGETSTATE"] + "()";
+	}
+}
+
+static std::string genGetCondition()
+{
+	if (bUseYYGetConditionNaked)
+	{
+		return mapCodeName["YYGETCONDITION"];
+	}
+	else
+	{
+		return mapCodeName["YYGETCONDITION"] + "()";
+	}
+}
+
+static void genSetCondition(std::ostream& o, uint ind, const std::string& newcond)
+{
+	if (bUseYYSetConditionParam)
+	{
+		o << indent(ind) << mapCodeName["YYSETCONDITION"] << "(" << condEnumPrefix << newcond << ");\n";
+	}
+	else
+	{
+		o << indent(ind) << replaceParam(mapCodeName["YYSETCONDITION"], yySetConditionParam, condEnumPrefix + newcond) << "\n";
+	}
+}
+
 static std::string space(uint this_label)
 {
 	int nl = next_label > 999999 ? 6 : next_label > 99999 ? 5 : next_label > 9999 ? 4 : next_label > 999 ? 3 : next_label > 99 ? 2 : next_label > 9 ? 1 : 0;
@@ -320,24 +397,28 @@ static void need(std::ostream &o, uint ind, uint n, bool & readCh, bool bSetMark
 	if (fFlag)
 	{
 		next_fill_index++;
-		o << indent(ind) << mapCodeName["YYSETSTATE"] << "(" << fillIndex << ");\n";
-	}
-
-	if (bUseYYFill)
-	{
-		if (n == 1)
+		if (bUseYYSetStateParam)
 		{
-			o << indent(ind) << "if(" << mapCodeName["YYLIMIT"] << " == " << mapCodeName["YYCURSOR"] << ") " << mapCodeName["YYFILL"];
+			o << indent(ind) << mapCodeName["YYSETSTATE"] << "(" << fillIndex << ");\n";
 		}
 		else
 		{
-			o << indent(ind) << "if((" << mapCodeName["YYLIMIT"] << " - " << mapCodeName["YYCURSOR"] << ") < " << n << ") " << mapCodeName["YYFILL"];
+			o << indent(ind) << replaceParam(mapCodeName["YYSETSTATE"], yySetStateParam, fillIndex) << "\n";
 		}
-		if (bUseYYFillParam)
+	}
+
+	if (bUseYYFill && n > 0)
+	{
+		if (n == 1)
 		{
-			o << "(" << n << ")";
+			o << indent(ind) << "if(" << mapCodeName["YYLIMIT"] << " == " << mapCodeName["YYCURSOR"] << ") ";
+			genYYFill(o, ind, n);
+		}
+		else
+		{
+			o << indent(ind) << "if((" << mapCodeName["YYLIMIT"] << " - " << mapCodeName["YYCURSOR"] << ") < " << n << ") ";
+			genYYFill(o, ind, n);
 		}
-		o << ";\n";
 	}
 
 	if (fFlag)
@@ -345,15 +426,18 @@ static void need(std::ostream &o, uint ind, uint n, bool & readCh, bool bSetMark
 		o << mapCodeName["yyFillLabel"] << fillIndex << ":\n";
 	}
 
-	if (bSetMarker)
+	if (n > 0)
 	{
-		o << indent(ind) << mapCodeName["yych"] << " = " << yychConversion << "*(" << mapCodeName["YYMARKER"] << " = " << mapCodeName["YYCURSOR"] << ");\n";
+		if (bSetMarker)
+		{
+			o << indent(ind) << mapCodeName["yych"] << " = " << yychConversion << "*(" << mapCodeName["YYMARKER"] << " = " << mapCodeName["YYCURSOR"] << ");\n";
+		}
+		else
+		{
+			o << indent(ind) << mapCodeName["yych"] << " = " << yychConversion << "*" << mapCodeName["YYCURSOR"] << ";\n";
+		}
+		readCh = false;
 	}
-	else
-	{
-		o << indent(ind) << mapCodeName["yych"] << " = " << yychConversion << "*" << mapCodeName["YYCURSOR"] << ";\n";
-	}
-	readCh = false;
 }
 
 void Match::emit(std::ostream &o, uint ind, bool &readCh) const
@@ -405,7 +489,7 @@ void Enter::emit(std::ostream &o, uint ind, bool &readCh) const
 
 void Initial::emit(std::ostream &o, uint ind, bool &readCh) const
 {
-	if (!startLabelName.empty())
+	if (!cFlag && !startLabelName.empty())
 	{
 		o << startLabelName << ":\n";
 	}
@@ -422,7 +506,7 @@ void Initial::emit(std::ostream &o, uint ind, bool &readCh) const
 		}
 	}
 
-	if (vUsedLabels.count(label))
+	if (!cFlag && vUsedLabels.count(label))
 	{
 		o << labelPrefix << label << ":\n";
 	}
@@ -593,6 +677,11 @@ void Rule::emit(std::ostream &o, uint ind, bool &) const
 		o << indent(ind) << mapCodeName["YYCURSOR"] << " = " << mapCodeName["YYCTXMARKER"] << ";\n";
 	}
 
+	if (rule->code->newcond)
+	{
+		genSetCondition(o, ind, rule->code->newcond->to_string());
+	}
+
 	RuleLine rl(*rule);
 
 	o << file_info(sourceFileInfo, &rl);
@@ -1386,15 +1475,16 @@ void DFA::findBaseState()
 	delete [] span;
 }
 
-void DFA::emit(std::ostream &o, uint ind)
+void DFA::emit(std::ostream &o, uint& ind, const RegExpMap* specMap, const std::string& condName, bool isLastCond, bool& bPrologBrace)
 {
 	State *s;
-	uint i, bitmap_brace = 0;
+	uint i;
 
 	findSCCs();
 	head->link = head;
 
 	uint nRules = 0;
+	bool bProlog = (!cFlag || !bWroteCondCheck);
 
 	for (s = head; s; s = s->next)
 	{
@@ -1521,13 +1611,6 @@ void DFA::emit(std::ostream &o, uint ind)
 	delete head->action;
 	head->action = NULL;
 
-	if (bFlag)
-	{
-		o << indent(ind++) << "{\n";
-		bitmap_brace = 1;
-		BitMap::gen(o, ind, lbChar, ubChar <= 256 ? ubChar : 256);
-	}
-
 	bUsedYYAccept = false;
 	
 	uint start_label = next_label;
@@ -1565,24 +1648,75 @@ void DFA::emit(std::ostream &o, uint ind)
 	next_fill_index = save_fill_index;
 
 	// Generate prolog
-	o << "\n" << outputFileInfo;
-	o << indent(ind++) << "{\n";
-
-	if (!fFlag)
+	if (bProlog)
 	{
-		o << indent(ind) << mapCodeName["YYCTYPE"] << " " << mapCodeName["yych"] << ";\n";
-		if (bUsedYYAccept)
+		o << "\n" << outputFileInfo;
+		if ((!fFlag && bUsedYYAccept)
+		||  (!fFlag && bEmitYYCh)
+		||  (bFlag && !cFlag && BitMap::first)
+		||  (cFlag && !bWroteCondCheck && gFlag && !specMap->empty())
+		||  (fFlag && !bWroteGetState && gFlag)
+		)
 		{
-			o << indent(ind) << "unsigned int "<< mapCodeName["yyaccept"] << " = 0;\n";
+			bPrologBrace = true;
+			o << indent(ind++) << "{\n";
+		}
+		else if (ind == 0)
+		{
+			ind = 1;
+		}
+	
+		if (!fFlag)
+		{
+			if (bEmitYYCh)
+			{
+				o << indent(ind) << mapCodeName["YYCTYPE"] << " " << mapCodeName["yych"] << ";\n";
+			}
+			if (bUsedYYAccept)
+			{
+				o << indent(ind) << "unsigned int "<< mapCodeName["yyaccept"] << " = 0;\n";
+			}
+		}
+		else
+		{
+			o << "\n";
 		}
 	}
-	else
+	if (bFlag && !cFlag)
 	{
-		o << "\n";
+		BitMap::gen(o, ind, lbChar, ubChar <= 256 ? ubChar : 256);
+	}
+	if (bProlog)
+	{
+		genCondTable(o, ind, *specMap);
+		genGetStateGoto(o, ind, start_label);
+		if (cFlag)
+		{
+			if (vUsedLabels.count(start_label))
+			{
+				o << labelPrefix << start_label << ":\n";
+			}
+			if (!startLabelName.empty())
+			{
+				o << startLabelName << ":\n";
+			}
+		}
+		genCondGoto(o, ind, *specMap);
 	}
 
-	genGetState(o, ind, start_label);
+	if (cFlag && !condName.empty())
+	{
+		// TODO: Drop marker
+		o << "/* *********************************** */\n";
+		o << condPrefix << condName << ":\n";
+	}
+	if (cFlag && bFlag && BitMap::first)
+	{
+		o << indent(ind++) << "{\n";
+		BitMap::gen(o, ind, lbChar, ubChar <= 256 ? ubChar : 256);
+	}
 
+	// TODO: Shouldn't labels 0 and 1 be variable?
 	if (vUsedLabels.count(1))
 	{
 		vUsedLabels.insert(0);
@@ -1597,9 +1731,12 @@ void DFA::emit(std::ostream &o, uint ind)
 		s->go.genGoto(o, ind, s, s->next, readCh);
 	}
 
+	if (cFlag && bFlag && BitMap::first)
+	{
+		o << indent(--ind) << "}\n";
+	}
 	// Generate epilog
-	o << indent(--ind) << "}\n";
-	if (bitmap_brace)
+	if ((!cFlag || isLastCond) && bPrologBrace)
 	{
 		o << indent(--ind) << "}\n";
 	}
@@ -1617,28 +1754,96 @@ void DFA::emit(std::ostream &o, uint ind)
 	bUseStartLabel = false;
 }
 
-void genGetState(std::ostream &o, uint& ind, uint start_label)
+void genGetStateGotoSub(std::ostream &o, uint ind, uint start_label, int cMin, int cMax)
+{
+	if (cMin == cMax)
+	{
+		if (cMin == -1)
+		{
+			o << indent(ind) << "goto " << labelPrefix << start_label << ";\n";
+		}
+		else
+		{
+			o << indent(ind) << "goto " << mapCodeName["yyFillLabel"] << cMin << ";\n";
+		}
+	}
+	else
+	{
+		int cMid = cMin + ((cMax - cMin + 1) / 2);
+
+		o << indent(ind) << "if (" << genGetState() << " < " << cMid << ") {\n";
+		genGetStateGotoSub(o, ind + 1, start_label, cMin, cMid - 1);
+		o << indent(ind) << "} else {\n";
+		genGetStateGotoSub(o, ind + 1, start_label, cMid, cMax);
+		o << indent(ind) << "}\n";
+	}
+}
+
+void genGetStateGoto(std::ostream &o, uint& ind, uint start_label)
 {
 	if (fFlag && !bWroteGetState)
 	{
 		vUsedLabels.insert(start_label);
-		o << indent(ind) << "switch(" << mapCodeName["YYGETSTATE"] << "()) {\n";
-		if (bUseStateAbort)
+		if (gFlag)
 		{
-			o << indent(ind) << "default: abort();\n";
-			o << indent(ind) << "case -1: goto " << labelPrefix << start_label << ";\n";
+			o << indent(ind++) << "static void *" << mapCodeName["yystable"] << "[" << "] = {\n";
+
+			for (size_t i=0; i<last_fill_index; ++i)
+			{
+				o << indent(ind) << "&&" << mapCodeName["yyFillLabel"] << i << ",\n";
+			}
+
+			o << indent(--ind) << "};\n";
+			o << "\n";
+
+			o << indent(ind) << "if(" << genGetState();
+			if (bUseStateAbort)
+			{
+				o << " == -1) {";
+				++ind;
+			}
+			else
+			{
+				o << " < 0)";
+			}
+			o << " goto " << labelPrefix << start_label << ";\n";
+			if (bUseStateAbort)
+			{
+				o << indent(--ind) << "} else if (" << genGetState() << " < -1) {\n";
+				o << indent(++ind) << "abort();\n";
+				o << indent(--ind) << "}\n";
+			}
+
+			o << indent(ind) << "goto *" << mapCodeName["yystable"] << "[" << genGetState() << "];\n";
+		}
+		else if (bFlag)
+		{
+			genGetStateGotoSub(o, ind, start_label, -1, last_fill_index-1);
+			if (bUseStateAbort)
+			{
+				o << indent(ind) << "abort();\n";
+			}
 		}
 		else
 		{
-			o << indent(ind) << "default: goto " << labelPrefix << start_label << ";\n";
+			o << indent(ind) << "switch(" << genGetState() << ") {\n";
+			if (bUseStateAbort)
+			{
+				o << indent(ind) << "default: abort();\n";
+				o << indent(ind) << "case -1: goto " << labelPrefix << start_label << ";\n";
+			}
+			else
+			{
+				o << indent(ind) << "default: goto " << labelPrefix << start_label << ";\n";
+			}
+	
+			for (size_t i=0; i<last_fill_index; ++i)
+			{
+				o << indent(ind) << "case " << i << ": goto " << mapCodeName["yyFillLabel"] << i << ";\n";
+			}
+	
+			o << indent(ind) << "}\n";
 		}
-
-		for (size_t i=0; i<last_fill_index; ++i)
-		{
-			o << indent(ind) << "case " << i << ": goto " << mapCodeName["yyFillLabel"] << i << ";\n";
-		}
-
-		o << indent(ind) << "}\n";
 		if (bUseStateNext)
 		{
 			o << mapCodeName["yyNext"] << ":\n";
@@ -1647,6 +1852,118 @@ void genGetState(std::ostream &o, uint& ind, uint start_label)
 	}
 }
 
+void genCondTable(std::ostream &o, uint ind, const RegExpMap& specMap)
+{
+	if (cFlag && !bWroteCondCheck && gFlag && specMap.size())
+	{
+		RegExpIndices  vCondList(specMap.size());
+
+		for(RegExpMap::const_iterator itSpec = specMap.begin(); itSpec != specMap.end(); ++itSpec)
+		{
+			vCondList[itSpec->second.first] = itSpec->first;
+		}
+
+		o << indent(ind++) << "static void *" << mapCodeName["yyctable"] << "[" << specMap.size() << "] = {\n";
+
+		for(RegExpIndices::const_iterator it = vCondList.begin(); it != vCondList.end(); ++it)
+		{
+			o << indent(ind) << "&&" << condPrefix << *it << ",\n";
+		}
+		o << indent(--ind) << "};\n";
+	}
+}
+
+static void genCondGotoSub(std::ostream &o, uint ind, RegExpIndices& vCondList, uint cMin, uint cMax)
+{
+	if (cMin == cMax)
+	{
+		o << indent(ind) << "goto " << condPrefix << vCondList[cMin] << ";\n";
+	}
+	else
+	{
+		uint cMid = cMin + ((cMax - cMin + 1) / 2);
+
+		o << indent(ind) << "if (" << genGetCondition() << " < " << cMid << ") {\n";
+		genCondGotoSub(o, ind + 1, vCondList, cMin, cMid - 1);
+		o << indent(ind) << "} else {\n";
+		genCondGotoSub(o, ind + 1, vCondList, cMid, cMax);
+		o << indent(ind) << "}\n";
+	}
+}
+
+void genCondGoto(std::ostream &o, uint ind, const RegExpMap& specMap)
+{
+	if (cFlag && !bWroteCondCheck && specMap.size())
+	{
+		if (gFlag)
+		{
+			o << indent(ind) << "goto *" << mapCodeName["yyctable"] << "[" << genGetCondition() << "];\n";
+		}
+		else
+		{
+			if (sFlag)
+			{
+				RegExpIndices  vCondList(specMap.size());
+			
+				for(RegExpMap::const_iterator it = specMap.begin(); it != specMap.end(); ++it)
+				{
+					vCondList[it->second.first] = it->first;
+				}
+				genCondGotoSub(o, ind, vCondList, 0, vCondList.size() - 1);
+			}
+			else
+			{
+				o << indent(ind) << "switch(" << genGetCondition() << ") {\n";
+	
+				for(RegExpMap::const_iterator it = specMap.begin(); it != specMap.end(); ++it)
+				{
+					o << indent(ind) << "case " << condEnumPrefix << it->first << ": goto " << condPrefix << it->first << ";\n";
+				}
+				o << indent(ind) << "}\n";
+			}
+		}
+		bWroteCondCheck = true;
+	}
+}
+
+void genTypes(std::string& o, uint ind, const RegExpMap& specMap)
+{
+	o.clear();
+
+	o += indent(ind++) + "enum " + mapCodeName["YYCONDTYPE"] + " {\n";
+
+	RegExpIndices  vCondList(specMap.size());
+
+	for(RegExpMap::const_iterator itSpecMap = specMap.begin(); itSpecMap != specMap.end(); ++itSpecMap)
+	{
+		vCondList[itSpecMap->second.first] = itSpecMap->first;
+	}
+
+	for(RegExpIndices::const_iterator itCondType = vCondList.begin(); itCondType != vCondList.end(); ++itCondType)
+	{
+		o += indent(ind) + condEnumPrefix + *itCondType + ",\n";
+	}
+
+	o += indent(--ind) + "};\n";
+}
+
+void genHeader(std::ostream &o, uint ind, const RegExpMap& specMap)
+{
+	o << "/* Generated by re2c " PACKAGE_VERSION;
+	if (!bNoGenerationDate)
+	{
+		o << " on ";
+		time_t now = time(&now);
+		o.write(ctime(&now), 24);
+	}
+	o << " */\n";
+	o << headerFileInfo;
+	o << "\n";
+	// now the type(s)
+	genTypes(typesInline, ind, specMap);
+	o << typesInline;
+}
+
 std::ostream& operator << (std::ostream& o, const file_info& li)
 {
 	if (li.ln && !iFlag)
@@ -1713,9 +2030,28 @@ void Scanner::config(const Str& cfg, int num)
 			yychConversion  = "";
 		}
 	}
+	else if (cfg.to_string() == "yych:emit")
+	{
+		bEmitYYCh = num != 0;
+	}
+	else if (cfg.to_string() == "define:YYFILL:naked")
+	{
+		bUseYYFillNaked = num != 0;
+	}
+	else if (cfg.to_string() == "define:YYGETCONDITION:naked")
+	{
+		bUseYYGetConditionNaked = num != 0;
+	}
+	else if (cfg.to_string() == "define:YYGETSTATE:naked")
+	{
+		bUseYYGetStateNaked = num != 0;
+	}
 	else
 	{
-		fatal("unrecognized configuration name or illegal integer value");
+		std::string msg = "unrecognized configuration name '";
+		msg += cfg.to_string();
+		msg += "' or illegal integer value";
+		fatal(msg.c_str());
 	}
 }
 
@@ -1730,15 +2066,20 @@ void Scanner::config(const Str& cfg, const Str& val)
 		mapVariableKeys.insert("variable:yyaccept");
 		mapVariableKeys.insert("variable:yybm");
 		mapVariableKeys.insert("variable:yych");
+		mapVariableKeys.insert("variable:yyctable");
+		mapVariableKeys.insert("variable:yystable");
 		mapVariableKeys.insert("variable:yytarget");
+		mapDefineKeys.insert("define:YYCONDTYPE");
 		mapDefineKeys.insert("define:YYCTXMARKER");
 		mapDefineKeys.insert("define:YYCTYPE");
 		mapDefineKeys.insert("define:YYCURSOR");
 		mapDefineKeys.insert("define:YYDEBUG");
 		mapDefineKeys.insert("define:YYFILL");
+		mapDefineKeys.insert("define:YYGETCONDITION");
 		mapDefineKeys.insert("define:YYGETSTATE");
 		mapDefineKeys.insert("define:YYLIMIT");
 		mapDefineKeys.insert("define:YYMARKER");
+		mapDefineKeys.insert("define:YYSETCONDITION");
 		mapDefineKeys.insert("define:YYSETSTATE");
 		mapLabelKeys.insert("label:yyFillLabel");
 		mapLabelKeys.insert("label:yyNext");
@@ -1770,13 +2111,36 @@ void Scanner::config(const Str& cfg, const Str& val)
 	{
 		labelPrefix = strVal;
 	}
+	else if (cfg.to_string() == "condprefix")
+	{
+		condPrefix = strVal;
+	}
+	else if (cfg.to_string() == "condenumprefix")
+	{
+		condEnumPrefix = strVal;
+	}
+	else if (cfg.to_string() == "define:YYFILL@len")
+	{
+		yyFillLength = strVal;
+		bUseYYFillParam = false;
+	}
+	else if (cfg.to_string() == "define:YYSETCONDITION@cond")
+	{
+		yySetConditionParam = strVal;
+		bUseYYSetConditionParam = false;
+	}
+	else if (cfg.to_string() == "define:YYSETSTATE@state")
+	{
+		yySetStateParam = strVal;
+		bUseYYSetStateParam = false;
+	}
 	else if (mapVariableKeys.find(cfg.to_string()) != mapVariableKeys.end())
     {
     	if (bFirstPass && !mapCodeName.insert(
     			std::make_pair(cfg.to_string().substr(sizeof("variable:") - 1), strVal)
     			).second)
     	{
-			fatal("variable already being used and cannot be changed");
+			fatalf("variable '%s' already being used and cannot be changed", cfg.to_string().c_str());
     	}
     }
 	else if (mapDefineKeys.find(cfg.to_string()) != mapDefineKeys.end())
@@ -1785,7 +2149,7 @@ void Scanner::config(const Str& cfg, const Str& val)
     			std::make_pair(cfg.to_string().substr(sizeof("define:") - 1), strVal)
     			).second)
     	{
- 			fatal("define already being used and cannot be changed");
+ 			fatalf("define '%s' already being used and cannot be changed", cfg.to_string().c_str());
  		}
     }
 	else if (mapLabelKeys.find(cfg.to_string()) != mapLabelKeys.end())
@@ -1794,12 +2158,15 @@ void Scanner::config(const Str& cfg, const Str& val)
     			std::make_pair(cfg.to_string().substr(sizeof("label:") - 1), strVal)
     			).second)
     	{
-			fatal("label already being used and cannot be changed");
+			fatalf("label '%s' already being used and cannot be changed", cfg.to_string().c_str());
     	}
     }
 	else
 	{
-		fatal("unrecognized configuration name or illegal string value");
+		std::string msg = "unrecognized configuration name '";
+		msg += cfg.to_string();
+		msg += "' or illegal string value";
+		fatal(msg.c_str());
 	}
 }
 
