@@ -1,4 +1,4 @@
-/* $Id: dfa.h 733 2007-05-02 20:19:32Z helly $ */
+/* $Id: dfa.h 750 2007-05-06 22:55:26Z helly $ */
 #ifndef _dfa_h
 #define _dfa_h
 
@@ -266,7 +266,7 @@ public:
 
 	void findSCCs();
 	void findBaseState();
-	void emit(std::ostream&, uint);
+	void emit(std::ostream&, uint&, const RegExpMap*, const std::string&, bool, bool&);
 
 	friend std::ostream& operator<<(std::ostream&, const DFA&);
 	friend std::ostream& operator<<(std::ostream&, const DFA*);
