@@ -1,4 +1,4 @@
-/* $Id: globals.h 733 2007-05-02 20:19:32Z helly $ */
+/* $Id: globals.h 727 2007-05-01 14:55:50Z helly $ */
 #ifndef	_globals_h
 #define	_globals_h
 
@@ -14,14 +14,17 @@ namespace re2c
 
 extern file_info sourceFileInfo;
 extern file_info outputFileInfo;
+extern file_info headerFileInfo;
 
 extern bool bFlag;
+extern bool cFlag;
 extern bool dFlag;
 extern bool eFlag;
 extern bool fFlag;
 extern bool gFlag;
 extern bool iFlag;
 extern bool sFlag;
+extern bool tFlag;
 extern bool uFlag;
 extern bool wFlag;
 
@@ -38,7 +41,12 @@ extern bool bUsedYYMarker;
 extern bool bUseStartLabel;
 extern std::string startLabelName;
 extern std::string labelPrefix;
+extern std::string condPrefix;
+extern std::string condEnumPrefix;
 extern std::string yychConversion;
+extern std::string yyFillLength;
+extern std::string yySetConditionParam;
+extern std::string yySetStateParam;
 extern uint maxFill;
 extern uint next_label;
 extern uint cGotoThreshold;
@@ -47,11 +55,18 @@ extern uint cGotoThreshold;
 extern uint topIndent;
 extern std::string indString;
 extern bool yybmHexTable;
+extern bool bEmitYYCh;
 extern bool bUseStateAbort;
 extern bool bUseStateNext;
-extern bool bWroteGetState;
 extern bool bUseYYFill;
 extern bool bUseYYFillParam;
+extern bool bUseYYFillNaked;
+extern bool bUseYYSetConditionParam;
+extern bool bUseYYGetConditionNaked;
+extern bool bUseYYSetStateParam;
+extern bool bUseYYGetStateNaked;
+extern bool bWroteGetState;
+extern bool bWroteCondCheck;
 
 extern uint asc2ebc[256];
 extern uint ebc2asc[256];
@@ -61,7 +76,8 @@ extern uint *xlat, *talx;
 extern uint next_fill_index;
 extern uint last_fill_index;
 extern std::set<uint> vUsedLabels;
-extern re2c::CodeNames mapCodeName;
+extern CodeNames mapCodeName;
+extern std::string typesInline;
 
 extern uint nRealChars;
 
