@@ -1,4 +1,4 @@
-/* $Id: main.cc 733 2007-05-02 20:19:32Z helly $ */
+/* $Id: main.cc 727 2007-05-01 14:55:50Z helly $ */
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #elif defined(_WIN32)
@@ -20,14 +20,17 @@ namespace re2c
 
 file_info sourceFileInfo;
 file_info outputFileInfo;
+file_info headerFileInfo;
 
 bool bFlag = false;
+bool cFlag = false;
 bool dFlag = false;
 bool eFlag = false;
 bool fFlag = false;
 bool gFlag = false;
 bool iFlag = false;
 bool sFlag = false;
+bool tFlag = false;
 bool uFlag = false;
 bool wFlag = false;
 
@@ -41,14 +44,25 @@ bool bUsedYYAccept  = false;
 bool bUsedYYMaxFill = false;
 bool bUsedYYMarker  = true;
 
+bool bEmitYYCh       = true;
 bool bUseStartLabel  = false;
 bool bUseStateNext   = false;
 bool bUseYYFill      = true;
 bool bUseYYFillParam = true;
+bool bUseYYFillNaked = false;
+bool bUseYYSetConditionParam = true;
+bool bUseYYGetConditionNaked = false;
+bool bUseYYSetStateParam = true;
+bool bUseYYGetStateNaked = false;
 
 std::string startLabelName;
 std::string labelPrefix("yy");
+std::string condPrefix("yyc_");
+std::string condEnumPrefix("yyc");
 std::string yychConversion("");
+std::string yyFillLength("@@");
+std::string yySetConditionParam("@@");
+std::string yySetStateParam("@@");
 uint maxFill = 1;
 uint next_label = 0;
 uint cGotoThreshold = 9;
@@ -58,13 +72,15 @@ std::string indString("\t");
 bool yybmHexTable = false;
 bool bUseStateAbort = false;
 bool bWroteGetState = false;
+bool bWroteCondCheck = false;
 
 uint nRealChars = 256;
 
 uint next_fill_index = 0;
 uint last_fill_index = 0;
 std::set<uint> vUsedLabels;
-re2c::CodeNames mapCodeName;
+CodeNames mapCodeName;
+std::string typesInline;
 
 free_list<RegExp*> RegExp::vFreeList;
 free_list<Range*>  Range::vFreeList;
@@ -78,6 +94,7 @@ static const mbo_opt_struct OPTIONS[] =
 {
 	mbo_opt_struct('?', 0, "help"),
 	mbo_opt_struct('b', 0, "bit-vectors"),
+	mbo_opt_struct('c', 0, "start-conditions"),
 	mbo_opt_struct('d', 0, "debug-output"),
 	mbo_opt_struct('e', 0, "ecb"),
 	mbo_opt_struct('f', 0, "storable-state"),
@@ -86,6 +103,7 @@ static const mbo_opt_struct OPTIONS[] =
 	mbo_opt_struct('i', 0, "no-debug-info"),
 	mbo_opt_struct('o', 1, "output"),
 	mbo_opt_struct('s', 0, "nested-ifs"),
+	mbo_opt_struct('t', 1, "type-header"),
 	mbo_opt_struct('u', 0, "unicode"),
 	mbo_opt_struct('v', 0, "version"),
 	mbo_opt_struct('V', 0, "vernum"),
@@ -106,6 +124,8 @@ static void usage()
 	"                        specifications with more than a few keywords (e.g. for\n"
 	"                        most programming languages).\n"
 	"\n"
+	"-c     --conditions     Require start conditions.\n"
+	"\n"
 	"-d     --debug-output   Creates a parser that dumps information during\n"
 	"                        about the current position and in which state the\n"
 	"                        parser is.\n"
@@ -126,6 +146,8 @@ static void usage()
 	"-s     --nested-ifs     Generate nested ifs for some switches. Many compilers\n"
 	"                        need this assist to generate better code.\n"
 	"\n"
+	"-t     --type-header    Generate a header file with type definitions.\n"
+	"\n"
 	"-u     --unicode        Implies -w but supports the full Unicode character set.\n"
 	"\n"
 	"-v     --version        Show version information.\n"
@@ -153,6 +175,7 @@ int main(int argc, char *argv[])
 	int c;
 	const char *sourceFileName = 0;
 	const char *outputFileName = 0;
+	const char *headerFileName = 0;
 
 	if (argc == 1)
 	{
@@ -170,6 +193,10 @@ int main(int argc, char *argv[])
 			sFlag = true;
 			break;
 
+			case 'c':
+			cFlag = true;
+			break;
+
 			case 'e':
 			xlat = asc2ebc;
 			talx = ebc2asc;
@@ -201,6 +228,11 @@ int main(int argc, char *argv[])
 			case 's':
 			sFlag = true;
 			break;
+
+			case 't':
+			tFlag = true;
+			headerFileName = opt_arg;
+			break;
 			
 			case '1':
 			bSinglePass = true;
@@ -260,6 +292,11 @@ int main(int argc, char *argv[])
 		std::cerr << "re2c: error: Cannot combine -1 and -b or -f switch\n";
 		return 1;
 	}
+	if (!cFlag && headerFileName)
+	{
+		std::cerr << "re2c: error: Can only output a header file when using -c switch\n";
+		return 2;
+	}
 
 	if (wFlag && eFlag)
 	{
@@ -308,6 +345,7 @@ int main(int argc, char *argv[])
 
 	// set up the output stream
 	re2c::ofstream_lc output;
+	re2c::ofstream_lc header;
 
 	if (outputFileName == 0 || (sourceFileName[0] == '-' && sourceFileName[1] == '\0'))
 	{
@@ -319,6 +357,15 @@ int main(int argc, char *argv[])
 		cerr << "re2c: error: cannot open " << outputFileName << "\n";
 		return 1;
 	}
+	if (headerFileName)
+	{
+		if (!header.open(headerFileName).is_open())
+		{
+			cerr << "re2c: error: cannot open " << headerFileName << "\n";
+			return 1;
+		}
+		headerFileInfo = file_info(headerFileName, &header);
+	}
 	Scanner scanner(source, output);
 	sourceFileInfo = file_info(sourceFileName, &scanner);
 	outputFileInfo = file_info(outputFileName, &output);
@@ -337,15 +384,16 @@ int main(int argc, char *argv[])
 
 		null_stream  null_dev;
 		Scanner null_scanner(null_source, null_dev);
-		parse(null_scanner, null_dev);
+		parse(null_scanner, null_dev, NULL);
 		next_label = 0;
 		next_fill_index = 0;
 		bWroteGetState = false;
+		bWroteCondCheck = false;
 		bUsedYYMaxFill = false;
 		bFirstPass = false;
 	}
 
 	bLastPass = true;
-	parse(scanner, output);
+	parse(scanner, output, header.is_open() ? &header : NULL);
 	return 0;
 }
