@@ -1,9 +1,13 @@
-/* $Id: re.h 733 2007-05-02 20:19:32Z helly $ */
+/* $Id: re.h 774 2007-07-10 19:21:49Z helly $ */
 #ifndef _re_h
 #define _re_h
 
 #include <iostream>
 #include <set>
+#include <map>
+#include <list>
+#include <vector>
+#include <string>
 #include "token.h"
 #include "ins.h"
 #include "globals.h"
@@ -260,6 +264,7 @@ public:
 	{
 		o << exp << "/" << ctx << ";";
 	}
+	RuleOp* copy(uint) const;
 
 #ifdef PEDANTIC
 private:
@@ -298,6 +303,8 @@ public:
 	const RuleOp& op;
 };
 
+RegExp *mkAlt(RegExp*, RegExp*);
+
 class AltOp: public RegExp
 {
 
@@ -483,9 +490,19 @@ private:
 #endif
 };
 
-extern void genCode(std::ostream&, RegExp*);
-extern void genCode(std::ostream&, uint, RegExp*);
-extern void genGetState(std::ostream&, uint&, uint);
+typedef std::set<std::string>           CondList;
+typedef std::pair<int, RegExp*>         NRegExp;
+typedef std::map<std::string, NRegExp>  RegExpMap;
+typedef std::vector<std::string>        RegExpIndices;
+typedef std::list<RuleOp*>              RuleOpList;
+
+extern void genCode(std::ostream&, uint&, RegExp*, const RegExpMap*, const std::string&, bool, bool&);
+extern void genGetStateGoto(std::ostream&, uint&, uint);
+extern void genCondTable(std::ostream&, uint, const RegExpMap&);
+extern void genCondGoto(std::ostream&, uint, const RegExpMap&);
+extern void genTypes(std::string&, uint, const RegExpMap&);
+extern void genHeader(std::ostream&, uint, const RegExpMap&);
+
 extern RegExp *mkDiff(RegExp*, RegExp*);
 extern RegExp *mkAlt(RegExp*, RegExp*);
 
