@@ -1,4 +1,4 @@
-/* $Id: scanner.h 733 2007-05-02 20:19:32Z helly $ */
+/* $Id: scanner.h 745 2007-05-05 17:14:38Z helly $ */
 #ifndef _scanner_h
 #define	_scanner_h
 
@@ -17,7 +17,7 @@ class Scanner:
 private:
 	std::istream&	in;
 	std::ostream&   out;
-	char	*bot, *tok, *ptr, *cur, *pos, *lim, *top, *eof;
+	char	*bot, *tok, *ptr, *cur, *pos, *lim, *top, *eof, *ctx;
 	uint	tchar, tline, cline, iscfg;
 
 private:
@@ -32,6 +32,7 @@ public:
 	int echo();
 	int scan();
 	
+	void fatalf(const char*, ...) const;
 	void fatal(const char*) const;
 	void fatal(uint, const char*) const;
 
