@@ -1,4 +1,4 @@
-/* $Id: substr.h 733 2007-05-02 20:19:32Z helly $ */
+/* $Id: substr.h 762 2007-06-24 18:47:27Z helly $ */
 #ifndef _substr_h
 #define _substr_h
 
@@ -98,4 +98,8 @@ char *strndup(const char *str, size_t len);
 
 #endif
 
+#if defined(_MSC_VER) && !defined(vsnprintf)
+#define vsnprintf _vsnprintf
+#endif
+
 #endif
